import {WebWebSocketHandler} from "./WebWebSocketHandler";
import {Modal} from "./Modal";
import {WebHandler} from "./WebHandler";
import {Pane} from "./Pane";
import {WebSocketPayload} from "../WebSocketPayload";
import {WebSocketEvent} from "../WebSocketEvent";
import {ButtonType} from "./ButtonType";
import {Containers} from "./Containers";
import {Setup} from "./Setup";
import {RequestType} from "../RequestType";
import container from "../database/Container";
import {IContainer} from "../database/IContainer";
import {Settings} from "./Settings";

const main = document.getElementById("main");
const time = document.getElementById("right");

document.addEventListener("DOMContentLoaded", async () => {
    console.log("DOM Loaded");
    WebHandler.openPane(Pane.NONE);


    await connect();

    setupOnClickEvents();

    // Wait because sometimes the browser isn't ready for this dom update :(
    setTimeout(() => {
        WebWebSocketHandler.request(RequestType.CONTAINERS).then(Setup.onContainerUpdate);
    }, 1000);


    setInterval(() => {
        if(!time ) return;

        let currentDate = new Date();
        time.innerText = "" + (currentDate.getHours() < 10 ? "0" + currentDate.getHours() : currentDate.getHours()) + ":" + (currentDate.getMinutes() < 10 ? "0" + currentDate.getMinutes() : currentDate.getMinutes());
    }, 1000);

    WebWebSocketHandler.registerForEvent(WebSocketEvent.CONTAINERS, (payload) => {
        console.log("Updating container list...")
        let container : IContainer;
        let bottomContainers = document.getElementById("menuContainers") as HTMLDivElement;
        bottomContainers.innerHTML = "";
        for( container of payload.data )
        {
            let containerDiv = document.createElement("div") as HTMLDivElement;
            containerDiv.classList.add("container");

            let span = document.createElement("span") as HTMLSpanElement;
            let pcnt = Math.round( container.filled * 100 / container.volume );
            if( !container.content )
                span.innerText = "-";
            else if( isNaN(pcnt) )
                span.innerText = "?%";
            else
                span.innerText = pcnt + "%";

            if( pcnt < 5 )
                containerDiv.style.backgroundColor = "red";
            else if ( pcnt < 15 )
                containerDiv.style.backgroundColor = "#ef4f00";
            else if ( pcnt < 30 )
                containerDiv.style.backgroundColor = "#ff5400";

            containerDiv.append(span);

            bottomContainers.append(containerDiv);
        }
    })
});

function setupOnClickEvents() {
    // Menu BTN
    const menuBtn = document.getElementById("menuBtn") as HTMLButtonElement;
    menuBtn.disabled = true;

    let timer = 0;
    function mouseDown() {
        timer = Date.now();
    }
    function mouseUp() {
        if( ( Date.now() - timer ) / 1000 > 5 )
            window.location.reload();
    }
    menuBtn.onmousedown = mouseDown;
    menuBtn.ontouchstart = mouseDown;
    menuBtn.onmouseup = mouseUp;
    menuBtn.ontouchend = mouseUp;

    function doMenu() {
        if (WebHandler.currentPane != Pane.MENU) {
            WebHandler.openPane(Pane.MENU);
        } else {
            WebHandler.openPane(Pane.MAIN);
        }
    }

    menuBtn.onclick = doMenu;

    // Menu buttons
    const menuContainersBtn = document.getElementById("menu_containers") as HTMLButtonElement;
    const menuStatsBtn = document.getElementById("menu_stats") as HTMLButtonElement;

    menuContainersBtn.onclick = Containers.openMenu;


    menuStatsBtn.onclick = async () => {

        let statsModal = new Modal("stats", "Statistiken");

        let txt = document.createElement("p");
        txt.innerHTML = `Folgende Statistiken wurden erfasst.`;
        statsModal.addContent(txt);

        let div = document.createElement("div");
        div.style.textAlign = "left";
        statsModal.addContent(div);

        let list = document.createElement("ul");
        div.append(list);

        statsModal.addContent(document.createElement("br"));
        statsModal.addButton(ButtonType.PRIMARY, "Schließen", () => statsModal.close());

        WebWebSocketHandler.request(RequestType.STATS).then((payload) => {
            let li = document.createElement("li");
            console.log(payload);
            li.innerText = "Cocktails ausgegeben: " + payload.data["drinks_finished"];
            list.append(li);

            li = document.createElement("li");
            li.innerText = "Häufigster Cocktail: " + payload.data["drink_most"];
            list.append(li);

            li = document.createElement("li");
            li.innerText = "Anzahl Ingredients: " + payload.data["count_ingredients"];
            list.append(li);

            li = document.createElement("li");
            li.innerText = "Anzahl Cocktails: " + payload.data["count_cocktails"];
            list.append(li);

        });

        await statsModal.open();
    };

    const menuSettingsBtn = document.getElementById("menu_settings") as HTMLButtonElement;
    const menuSetupBtn = document.getElementById("menu_setup") as HTMLButtonElement;

    menuSettingsBtn.onclick = () => WebHandler.openPane(Pane.SETTINGS);
    menuSetupBtn.onclick = () => {
        let payload = new WebSocketPayload(WebSocketEvent.SETUP, true);
        WebWebSocketHandler.send(payload);
    }


    Settings.addListeners();




}

let wsHandler;

function connect(): Promise<void> {
    return new Promise(resolve => {
        wsHandler = new WebWebSocketHandler(resolve);
    });

}
